"""
xkcd API Wrapper
~~~~~~~~~~~~~~~~~~~

A basic API wrapper for xkcd.

:copyright: (c) 2025 Omkaar
:license: MIT, see LICENSE for more details.
"""


__title__ = "xkcd.py"
__author__ = "Ombucha"
__license__ = "MIT"
__copyright__ = "Copyright 2025 Omkaar"
__version__ = "1.1.0"


from .comic import Comic
from .what_if import WhatIfArticle
